@@ -41,7 +41,8 @@
 //! struct Foo;
 //!
 //! impl Encoder<Foo> for u8 {
-//!     fn encode<W: Write>(&self, writer: &mut W, params: Foo) -> Result<()> {
+//!     type Error = std::io::Error;
+//!     fn encode(&self, writer: &mut impl Write, params: Foo) -> Result<()> {
 //!         writer.write_all(slice::from_ref(self))?;
 //!         Ok(())
 //!     }
@@ -67,7 +68,8 @@
 //! struct Foo;
 //!
 //! impl Decoder<Foo> for u8 {
-//!     fn decode<R: Read>(reader: &mut R, params: Foo) -> Result<Self> {
+//!     type Error = std::io::Error;
+//!     fn decode(reader: &mut impl Read, params: Foo) -> Result<Self> {
 //!         let mut byte = 0u8;
 //!         reader.read_exact(slice::from_mut(&mut byte))?;
 //!         Ok(byte)
@@ -78,16 +80,30 @@
 //! assert_eq!(u8::decode(&mut buf.as_ref(), Foo).unwrap(), 7u8);
 //! ```
 
-use std::io::{Error, Read, Write};
+#![cfg_attr(has_associated_type_defaults, feature(associated_type_defaults))]
+
+use std::io::{Read, Write};
 
 /// Trait used to express encoding relationships.
-pub trait Encoder<T, E=Error> {
+pub trait Encoder<T> {
+    #[cfg(has_associated_type_defaults)]
+    type Error = std::io::Error;
+
+    #[cfg(not(has_associated_type_defaults))]
+    type Error;
+
     /// Encodes to the writer with the given parameters.
-    fn encode<W: Write>(&self, writer: &mut W, params: T) -> Result<(), E>;
+    fn encode(&self, writer: &mut impl Write, params: T) -> Result<(), Self::Error>;
 }
 
 /// Trait used to express decoding relationships.
-pub trait Decoder<T, E=Error>: Sized {
+pub trait Decoder<T>: Sized {
+    #[cfg(has_associated_type_defaults)]
+    type Error = std::io::Error;
+
+    #[cfg(not(has_associated_type_defaults))]
+    type Error;
+
     /// Decodes from the reader with the given parameters.
-    fn decode<R: Read>(reader: &mut R, params: T) -> Result<Self, E>;
+    fn decode(reader: &mut impl Read, params: T) -> Result<Self, Self::Error>;
 }
